
/* tkstat: read netphone logfiles and print statistics
   Maik Musall, 1995-2000 */

#include <stdio.h>
#include <stdlib.h>
#include <ncurses.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <ctype.h>
#include <signal.h>

#define version "1.20"
#define verdate "14.06.2000"


/* default values */

#define defcfgfilename "/etc/tkstat.cfg"
#define defnormalfilename "/var/log/netphone"
#define defdebugfilename "/var/log/netphtest"
#define defphofilename "/etc/netphnums"
#define defdenyfilename "/etc/netphdenynums"
#define defisdnlogfilename "/var/log/isdnlog.out"
#define debfilename "tkstat.debug"
#define defvorwahl "09131"
#define defpfennig 12
#define anzlrings 5
#define maxcallers 500

#define defname0 "Anlage"
#define defname1 "Bro"
#define defname2 "Maik"
#define defname3 "Felix"
#define defname4 "Faxmodem"
#define defname5 "Dummy"


enum nstellen { Anlage,
		Buero,
		Maik,
		Felix,
		Faxmodem,
		Dummy };

#define idle "Idle"
#define busy "Besetzt"
#define unreachable "Nicht erreichbar"
#define anwahl "Anwahl"
#define intern "Interngespraech"
#define rufint "Ruf von Intern"
#define rufamt "Ruf vom Amt"
#define rufuml "U-Ruf vom Amt"
#define extmit "Ext mit"
#define extvon "Ext von"
#define extueb "Ext ber "
#define offhook "Abgehoben"

#define ringint 1
#define ringext 2
#define calling 3
#define silence 0

#define no 0
#define yes 1
#define stdmsecs 500
#define fieldlength 16
#define maxnstnamelen 11

enum { city = 0, reg50, reg200, fern, euro1, euro2, welt1, welt2, welt3, welt4,
	cnetz, dnetz, enetz, cbox, anztarife};

extern time_t get_date ();

static int einhlen[anztarife][24];
static char fmeld[80];
static int uebnahm, debug, debdelay;
static int msecs;
static struct timeval delay;
static int continuous, timer;
static int timediff;
static int calldbsize, shownames, accepted, both, angen;
static char scrbuf[fieldlength+anzlrings][96];
static char logfilename[80];
static int compdial, lastcallmonth, denyentries;
static int uppernst, four, five, isdnlogging;
static int othercfgfile, defaultvalues;
static char cfgfilename[255], normalfilename[255], debugfilename[255];
static char phofilename[255], vorwahl[10], ilogfilename[255], denyfilename[255];
static int pfennig, skipcol, noclock, denydbsize, bellring, rangonce;
static FILE *debfile;
static char debs[120];
static char mailfilename[255];
static int staticmail, mailwaiting;

struct nstdata_t {
	char name[maxnstnamelen];
	char status[18];
	char lastiring[18];
	char lastering[18];
	char lastenum[22];
	char callingnr[25];
	time_t gbegin;
	int umleitruf;
	int ringing;
	struct {
		unsigned int date, units;
		char time[6], number[15];
		char duration[9], betrag[6];
	} lcall;
	unsigned int unitsum;
	struct {
		char time[8];
		char number[13];
		char acc;
	} lrings[anzlrings];
	int network;
};
static struct nstdata_t nstdats[6];

struct phonenr_t {
    char number[20], name[80];
/*  int network; */
} callers[maxcallers];

struct {
	int network;
	char nwname[16];
} networks[] = {
	{0,  "Default"},
	{11, "o.tel.o"},
	{13, "Tele2"},
	{19, "Mobilcom"},
	{24, "Super24"},
	{30, "TelDaFax"},
	{33, "Telekom"},
	{50, "Talkline"},
	{66, "Interoute"},
	{70, "Arcor"},
	{90, "Viag"},
};

static int NUMBEROFNETWORKS; /* wird in initdata() initialisiert */

/* als struct, falls nochmal was dazukommt */
struct denynr_t {
    char number[20];
} noentrynums[maxcallers];


void out(char s[])
{
	if (debug)
		if (debdelay) {
			fprintf(stderr, "DEBUG: <%s>\n", s);
			sleep(1);
		} else
			if (continuous) {
				fprintf(debfile, "DEBUG: <%s>\n", s);
				fflush(debfile);
			} else
				printf("DEBUG: <%s>\n", s);
}


gettcentry(char entry[], char such[])
{
	char *envstr, cmd[3];
	int i, j;
	
	envstr = (char *)getenv("TERMCAP");
	i = 0;
	cmd[2]='\0';
	do {
		while(envstr[i] != ':') i++;
		j=0;
		cmd[0] = envstr[++i];
		cmd[1] = envstr[++i];
		i+=2; while(envstr[i] != ':')
			entry[j++] = envstr[i++];
		entry[j] = '\0';
	} while (strncmp(cmd, such, 2));
/*
	for (i=0; i<strlen(entry); i++)
		if((entry[i]=='\\') && (entry[i+1]=='E')) {
			entry[i]=27;
			entry[strlen(entry)-2]='\0';
		}
*/
}


listentry(unsigned int app, char datum[15])
{
	int i, j;
	char mon[3], outs[1024];
	int newentry = 0, nolistentry = no;

if(debug) sprintf(outs, "entering listentry(), denydbsize=%d", denydbsize);
out(outs);
	for(i=0; i<denydbsize && !nolistentry; i++) {
if(debug) sprintf(outs, "comparing '%s' against: '%s'", nstdats[app].callingnr, noentrynums[i].number);
out(outs);
	    if(!strncmp(noentrynums[i].number, nstdats[app].callingnr, strlen(noentrynums[i].number)))
		nolistentry = yes;
	    if(!strncmp(noentrynums[i].number, vorwahl, strlen(vorwahl)))
	        if(!strncmp(noentrynums[i].number+strlen(vorwahl), nstdats[app].callingnr, strlen(nstdats[app].callingnr)))
	            nolistentry = yes;
	}
	if (nolistentry) {
out("nolistentry matched");
	    return(0);
	}
	
	for(i=0; i<anzlrings; i++)
		if(nstdats[app].lrings[i].time=="") {
			strcpy(nstdats[app].lrings[i].time, "000000");
			nstdats[app].lrings[i].time[0] = datum[4];
			nstdats[app].lrings[i].time[1] = datum[5];
			nstdats[app].lrings[i].time[2] = datum[7];
			nstdats[app].lrings[i].time[3] = datum[8];
			nstdats[app].lrings[i].time[4] = datum[10];
			nstdats[app].lrings[i].time[5] = datum[11];
			if(strlen(nstdats[app].callingnr) > 0)
				strncpy(nstdats[app].lrings[i].number, nstdats[app].callingnr, 17-strlen(nstdats[app].lrings[i].time));
			else
				strcpy(nstdats[app].lrings[i].number, "(taken)");
			nstdats[app].lrings[i].acc = (angen?'y':'n');
			newentry++;
		}
	if(!newentry) {            /* Ersteintrag */
		for(i=0; i<anzlrings-1; i++)
			nstdats[app].lrings[i] = nstdats[app].lrings[i+1];
		nstdats[app].lrings[anzlrings-1].time[0] = datum[4];
		nstdats[app].lrings[anzlrings-1].time[1] = datum[5];
		nstdats[app].lrings[anzlrings-1].time[2] = datum[7];
		nstdats[app].lrings[anzlrings-1].time[3] = datum[8];
		nstdats[app].lrings[anzlrings-1].time[4] = datum[10];
		nstdats[app].lrings[anzlrings-1].time[5] = datum[11];
		if(strlen(nstdats[app].callingnr) > 0)
			strncpy(nstdats[app].lrings[anzlrings-1].number, nstdats[app].callingnr, 12);
		else
			strcpy(nstdats[app].lrings[i].number, "(taken)");
		nstdats[app].lrings[i].acc = (angen?'y':'n');
	}
}


void reverse(char s[])
{
	int c, i, j;
	
	for (i=0, j = strlen(s)-1; i < j; i++, j--) {
		c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
}


void itoa(char s[], int n)
{
	int i, sign;
	
	if ((sign = n) < 0)
		n = -n;
	i = 0;
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	reverse(s);
}


void suchgst(char st[], unsigned int app)
{
	int i, nst = 0, anz = 0;
	
	for(i=1;i<=5;i++)
		if(nstdats[i].status==offhook) {
			anz++;
			nst=i;
		}
	if(anz>1)
		st="(unknown)";
	else
		itoa(st, nst);
}


int cmdnametoi(char command[])
{
	if (!strncmp(command, "OFFH", 4)) return 1;
	if (!strncmp(command, "ONHO", 4)) return 2;
	if (!strncmp(command, "RING", 4)) return 3;
	if (!strncmp(command, "CONN", 4)) return 4;
	if (!strncmp(command, "CALL", 4)) return 5;
	if (!strncmp(command, "BUSY", 4)) return 6;
	if (!strncmp(command, "NO A", 4)) return 7;
	return 0;
}


int fullcmdnametoi(char command[])
{
	if (!strcmp(command, "RING INTERN ENDE")) return 1;
	if (!strcmp(command, "RING END NEGATIV")) return 2;
	if (!strcmp(command, "RING INTERN")) return 3;
	if (!strcmp(command, "CONNECT INTERN")) return 4;
	if (!strcmp(command, rufint)) return 5;
	if (!strcmp(command, rufamt)) return 6;
	if (!strcmp(command, anwahl)) return 7;
	if (!strcmp(command, "RING END POSITIV")) return 8;
	return 0;
}


speaking()
{
	char s[80];
	int i;
	
	for(i=1; i<=uppernst; i++)
		if (i != skipcol) {
			strcpy(s, nstdats[i].status);
			if (!strcmp(s, intern) || !strcmp(s, extmit) || !strcmp(s, extvon))
				return 1;
		}
	return 0;
}


gesprdauer(char sout[], int app)
{
	char s[80], *zeit, sec[4], min[4];
	time_t when;
	struct tm *convtime;
	size_t maxlen = 18;
	int seconds, i, ilen;
	
	strcpy(s, nstdats[app].status);
	if (timer && (!strcmp(s, intern) || !strcmp(s, extmit) || !strcmp(s, extvon))) {
		time(&when);
		convtime = localtime (&when);
/*
		zeit = (char *)realloc(zeit, maxlen);
*/
		seconds = (int)difftime(mktime(convtime), nstdats[app].gbegin);
		if (seconds<0)
			timediff = 1 - seconds;
		seconds+=timediff;
		pzwei(sec, seconds%60);
		pzwei(min, seconds/60);
		sprintf(s, " %s:%s ", min, sec);
		ilen = 9 + (strlen(s)>7?0:1);
		for(i=ilen; i<=16; i++)
			sout[i] = s[i-ilen];
	}
}


char *nwctoname(int network)
{
	static char nwname[25];
	int i;
	
	strcpy(nwname, "unknown");
	for(i=0; i<NUMBEROFNETWORKS; i++) {
		if(networks[i].network == network) {
			strcpy(nwname, networks[i].nwname);
			break;
		}
	}
	return nwname;
}


void printstats()
{
	const char ast[] = "******************";
	const char spc[] = "                  ";
	const char sep[] = "|";
	char scr[fieldlength+anzlrings][96], dmbetr[7], vier[6];
	char tempa[25], temps[25], dummy[96], prev[25], timea[25];
	char numline[20], zeit[26], spstr[4], outstatus[25], fillbetr[4];
	char nwname[25];
	int i, j, found, dlen;
	int currnet;
	time_t when;
	size_t maxlen = 25;
	struct tm *convtime;
	struct stat mailfilestat;
	
	for(i=0; i<15+anzlrings; i++)
		strcpy(scr[i], "");
	strcpy(tempa, ast);
	strcat(tempa, sep);
	strcpy(temps, spc);
	strcat(temps, sep);
/* debdelay = yes; */ /* Debug-output delay */
	for(i=1; i<=uppernst; i++)
	  if (i != skipcol) {
		if (((i == 1) || ((skipcol == 1) && (i == 2))) && (noclock == 0)) {
			strcpy(timea, ast);
			strcat(timea, sep);
			time(&when);
			convtime = localtime (&when);
/*
			zeit = (char *)realloc(zeit, maxlen);
*/
			strcpy(zeit, "                        ");
			strftime(zeit, maxlen, "%H:%M", convtime);
			for(j=0; j<strlen(zeit); j++)
				timea[j+2] = zeit[j];
			timea[1] = timea[7] = ' ';
			gesprdauer(timea, i);
		} else {
			strcpy(timea, ast);
			strcat(timea, sep);
			gesprdauer(timea, i);
		}
		if ((i == 1) || ((skipcol == 1) && (i == 2))) {
			time(&when);
			if (when % 5 == 0) {
				stat (mailfilename, &mailfilestat);
				if (mailfilestat.st_mtime > mailfilestat.st_atime) {
					mailwaiting = yes;
				} else {
					mailwaiting = no;
				}
			}
			if (mailwaiting) {
				if ((when % 6 > 2) || staticmail) {
					timea[0] = 'M';
				} else {
					timea[0] = 'm';
				}
			} else {
				timea[0] = '*';
			}
		}
		strcat(scr[0], timea);
		strcat(scr[9], tempa);
		strcat(scr[13], tempa);
		strcat(scr[14+anzlrings], tempa);
		strcat(scr[2], temps);
		strcat(scr[5], temps);
		strcpy(dummy, spc);			/* Namenzeile */
		j = strlen(spc) - strlen(nstdats[i].name) - 4;
		dummy[j]='\0';
		sprintf(prev, " N%d%s%s %s", i, dummy, nstdats[i].name, sep);
		strcat(scr[1], prev);
		strcpy(dummy, spc);			/* Statuszeile */
		if(nstdats[i].umleitruf && !(strcmp(nstdats[i].status, rufamt)))
			strcpy(outstatus, rufuml);
		else {
			strcpy(outstatus, nstdats[i].status);
			/* wenn bellring auf nst und rufamt, noch EIN ctrl-g ausgeben */
			if((bellring == i) && (!strcmp(outstatus, rufamt)) && !rangonce) {
				printf("%c", 7);
				rangonce = yes;
			}
		}
		j = strlen(spc) - strlen(outstatus) - 2;
		dummy[j]='\0';
		/* Evtl. Netzbetreiber einblenden */
		if (strcmp(nstdats[i].status, extmit) && strcmp(nstdats[i].status, extvon) && strcmp(nstdats[i].status, anwahl) && strcmp(nstdats[i].status, rufamt))
			strcpy(nstdats[i].callingnr, "");
		currnet = numtoname(numline, nstdats[i].callingnr, 14);
		if(currnet!=0) nstdats[i].network = currnet; /* reset durch idle */

if(debug) {
sprintf(debs, "analyze currnet = %d", currnet);
out(debs);
sprintf(debs, "nstdats[i].network = %d", nstdats[i].network);
out(debs);
}
		if(nstdats[i].network != 0 && strlen(dummy)>3 && (!strcmp(outstatus, extmit) || !strcmp(outstatus, anwahl))) {
			dummy[1]='>';
			strcpy(nwname, nwctoname(nstdats[i].network));
			for(j=2; j<strlen(dummy) && j-2<strlen(nwname); j++)
				dummy[j]=nwname[j-2];
			/*strncpy(dummy+2, nwname, strlen(dummy)-3);*/
		}

		sprintf(prev, " %s%s %s", outstatus, dummy, sep);
		strcat(scr[3], prev);
		strcpy(dummy, spc);			/* Nummernzeile */
out("jetzt");
out(nstdats[i].status);
out(nstdats[i].callingnr);
		dummy[strlen(spc) - strlen(numline) - 2]='\0';
		sprintf(prev, " %s%s %s", numline, dummy, sep);
		strcat(scr[4], prev);		/* Uhrzeit/Gesamtbetrag */
		pz(dmbetr, nstdats[i].unitsum*pfennig);
		pvier(vier, nstdats[i].lcall.date);
		sprintf(prev, "%s %s %s %s", vier, nstdats[i].lcall.time, dmbetr, sep);
		strcat(scr[6], prev);
		if(nstdats[i].lcall.units > 99)
			strcpy(spstr, "");
		else
			strcpy(spstr, " ");
		if(strlen(nstdats[i].lcall.betrag)<5)
			strcpy(fillbetr, " ");
		else
			strcpy(fillbetr, "");
		sprintf(prev, "%s %s%s %2d%s%s",
		 nstdats[i].lcall.duration, fillbetr, nstdats[i].lcall.betrag,
		 nstdats[i].lcall.units, spstr, sep);
		strcat(scr[7], prev);
		strcpy(dummy, spc);			/* Telefonnummer */
		numtoname(numline, nstdats[i].lcall.number, 14); /* other networks kommen hier noch nicht vor */
		j = strlen(spc) - strlen(numline) - 4;
		dummy[j]='\0';
		sprintf(prev, " @ %s%s %s", numline, dummy, sep);
		strcat(scr[8], prev);
		sprintf(prev, "I %s %s",		/* Letzter Intern */
		 nstdats[i].lastiring, sep);
		strcat(scr[10], prev);
		sprintf(prev, "E %s %s",		/* Letzter Extern */
		 nstdats[i].lastering, sep);
		strcat(scr[11], prev);
		strcpy(dummy, spc);			/* Von Telefonnummer */
		numtoname(numline, nstdats[i].lastenum, 14);
		j = strlen(spc) - strlen(numline) - 4;
		dummy[j]='\0';
		sprintf(prev, "E= %s%s %s", numline, dummy, sep);
		strcat(scr[12], prev);
out("forj");
		for(j=0; j<anzlrings; j++) {
			strcpy(dummy, spc);		/* Listeneintrag */
if(debug) sprintf(debs, "NNNNN: %d", 17 - strlen(nstdats[i].lrings[j].time) - both);
out(debs);
			numtoname(numline, nstdats[i].lrings[j].number, 17 - strlen(nstdats[i].lrings[j].time) - both);
			dummy[strlen(spc) - strlen(numline) - strlen(nstdats[i].lrings[j].time) - 1 - both]='\0';
if(debug) sprintf(debs, "NNNNN: %d %d", strlen(numline), strlen(spc) - strlen(numline) - strlen(nstdats[i].lrings[j].time) - 1 - both);
out(debs);
			if (both)
				sprintf(prev, "%s%c %s%s%s", nstdats[i].lrings[j].time, nstdats[i].lrings[j].acc, numline, dummy, sep);
			else
				sprintf(prev, "%s %s%s%s", nstdats[i].lrings[j].time, numline, dummy, sep);
			strcat(scr[14+j], prev);
		}
out("afterforj");
	}
	
out("zw1");
	if (continuous) {
		for(i=0; i<fieldlength-1+anzlrings; i++) {
			sprintf(dummy, "%s\n", scr[i]);
			mvaddstr(i, 0, dummy);
		}
		refresh();
/*		system("tput cup 0 0"); */
	} else {
out("zw2");
out("foroutloop");
		for(i=0; i<fieldlength-1+anzlrings; i++)
			printf("%s\n", scr[i]);
	}
}


getlogdata()
{
	FILE *nplogf, *tmpfile, *ilogf;
	char inpline[80], day[3], month[4], datestr[15], dummyline[80];
	char dauer[9], nummer[30], betrag[6], einh[3], zeit[10];
	char command[30], nstueb[2], rest[40], *fileread, *isdnlogread;
	unsigned int app, dm, pf, i, j, br;
	time_t current, vergleich;

	if ((nplogf = fopen(logfilename, "r")) == NULL) {
		fprintf(stderr, "Error: logfile %s not found!\nExiting.\n", logfilename);
		exit(2);
	}
	if (isdnlogging)
		if ((ilogf = fopen(ilogfilename, "r")) == NULL) {
			fprintf(stderr, "Error: ISDN-Logfile %s not found! Ignoring switch -i\n", ilogfilename);
			isdnlogging = no;
		}
	if (isdnlogging)
		printf("ISDN Logfile currently not supported!  Sorry...\n");
	while(((fileread=fgets(inpline, 80, nplogf))!=NULL)||continuous) {
		if(continuous && (fileread == NULL)) {
out("printstatswhileloop");
			printstats();
			time(&current);
			while(((fgets(inpline, 80, nplogf))==NULL)) {
				/* (void) select(0, NULL, NULL, NULL, &delay); */
out("sleeping...");
				sleep(1);
				time(&vergleich); /* angezeigte Uhrzeit aktuell halten */
				if(current!=vergleich) {
					current=vergleich;
					printstats();
				}
				if(speaking()) printstats();
			}
out("out of sleep loop");
		}
		if(!strncmp(inpline, "App ", 4)) {
			sscanf(inpline, "App %d an %s vom %s %s %s DM %s\n",
			 &app, nummer, datestr, einh, betrag, dauer);
			strcpy(nstdats[app].lcall.duration, dauer);
			strcpy(nstdats[app].lcall.betrag, betrag);
			nstdats[app].lcall.units = atoi(einh);
			strncpy(nstdats[app].lcall.number, nummer, 14);
			nstdats[app].lcall.date = (datestr[0]-'0')*1000 + (datestr[1]-'0')*100 +
						  (datestr[3]-'0')*10 + datestr[4]-'0';
			for(i=9, j=0; i<14; i++, j++)
				nstdats[app].lcall.time[j] = datestr[i];
if(debug) sprintf(debs, "LLLLL: %d", lastcallmonth);
out(debs);
			if(nstdats[app].lcall.date%100 != lastcallmonth) {
				for(i=0; i<=5; i++)
					nstdats[i].unitsum = 0;
out("reset lastcallmonth");
				lastcallmonth = nstdats[app].lcall.date%100;
			}
if(debug) sprintf(debs, "LLLLL: %d", lastcallmonth);
out(debs);
			nstdats[app].unitsum+=nstdats[app].lcall.units;
		} else if((inpline[17] != 'L')&&(inpline[17] != '2')&&(inpline[17] != 'E')&&(inpline[17] != 'N')) {
			strncpy(datestr, inpline, 15);
			datestr[15]='\0';
			for(i=18, j=0; inpline[i]!='@'; i++,j++)
				command[j] = inpline[i];
			command[j] = '\0';
			app = inpline[++i] - 48;
if(debug) sprintf(debs, "DDDDD: %d", app);
out(debs);
out(command);
			if((inpline[17]=='O') && (inpline[18]=='K'))
				compdial = yes;
			else switch(cmdnametoi(command)) {
				case 1 : if ((strcmp(nstdats[app].status, rufint) != 0) && (strcmp(nstdats[app].status, rufamt) != 0) && (strcmp(nstdats[app].status, anwahl) != 0))
						strcpy(nstdats[app].status, offhook);
					 compdial = no;
					 break;
				case 2 : uebnahm = no;
					 strcpy(nstdats[app].status, idle);
					 rangonce = no;
					 nstdats[app].network = 0;
					 if (nstdats[app].ringing < ringext)
					 	strcpy(nstdats[app].callingnr, "");
					 timediff = 0;
					 break;
				case 3 : switch(fullcmdnametoi(command)) {
					case 1  : uebnahm = no;
						  strcpy(nstdats[app].status, idle);
						  rangonce = no;
						 nstdats[app].network = 0;
						  strcpy(nstdats[app].callingnr, "");
						  nstdats[app].ringing = silence;
						  break;
					case 2  : if ((!accepted) || both) {
						  	angen = no;
						  	listentry(app, datestr);
						  }
						  strcpy(nstdats[app].status, idle);
						  rangonce = no;
						  nstdats[app].network = 0;
						  strcpy(nstdats[app].callingnr, "");
						  nstdats[app].ringing = silence;
						  break;
					case 3  : strcpy(nstdats[app].status, rufint);
						  strncpy(nstdats[app].lastiring, inpline, 15);
out("lastiring Zuweisung");
out(nstdats[app].lastiring);
						  nstdats[app].ringing = ringint;
						  break;
					case 8 : strcpy(nstdats[app].status, idle);
						 rangonce = no;
						 nstdats[app].network = 0;
						 nstdats[app].ringing = silence;
						 uebnahm = app; break;
					default : if (!strcmp(nstdats[app].status, rufamt))
							nstdats[app].umleitruf = 1;
						  else
						  	nstdats[app].umleitruf = 0;
						  strcpy(nstdats[app].status, rufamt);
						  strncpy(nstdats[app].lastering, inpline, 15);
						  nstdats[app].ringing = ringext;
						  if(strcmp(command, "RING"))
							sscanf(command, "RING%s", nstdats[app].callingnr);
						  else
						  	strcpy(nstdats[app].callingnr, "(unknown)");
						  strcpy(nstdats[app].lastenum, nstdats[app].callingnr);
					} break;
				case 4 : nstdats[app].gbegin = get_date(datestr, NULL);
if (debug) sprintf(debs, "get_date says %ld for %s", nstdats[app].gbegin, datestr);
out(debs);
					 switch(fullcmdnametoi(command)) {
					case 4  : strcpy(nstdats[app].status, intern);
					          break;
					default : switch(fullcmdnametoi(nstdats[app].status)) {
							case 5 : strcpy(nstdats[app].status, intern);
								 suchgst(nstdats[app].callingnr, app);
								 break;
							case 6 : strcpy(nstdats[app].status, extvon);
							         if (accepted || both) {
							         	angen = yes;
							         	listentry(app, datestr);
							         }
								 break;
							case 7 : strcpy(nstdats[app].status, extmit); break;
							default: if(uebnahm!=0) {
									strcpy(nstdats[app].status, extueb);
									strcpy(nstdats[app].callingnr, nstdats[uebnahm].callingnr);
									itoa(nstueb, uebnahm);
									strcat(nstdats[app].status, nstueb);
								 }
								 switch(nstdats[app].ringing) {
								 	case ringint: strcpy(nstdats[app].status, intern);
										      suchgst(nstdats[app].callingnr, app);
										      break;
									case ringext: strcpy(nstdats[app].status, extvon);
										      break;
									case calling: strcpy(nstdats[app].status, extmit);
										      break;
								 }
						}
					 }
					 nstdats[app].ringing = silence;
					 break;
				case 5 : strcpy(nstdats[app].status, anwahl);
					 sscanf(command, "CALLING%s", nstdats[app].callingnr);
					 nstdats[app].ringing = calling;
					 break;
				case 6 : if (compdial && (app==1)) {
						uebnahm = no;
					 	strcpy(nstdats[app].status, idle);
					 	rangonce = no;
						nstdats[app].network = 0;
					 	strcpy(nstdats[app].callingnr, "");
						timediff = 0;
				            } else {
					     if(strcmp(nstdats[app].status, anwahl))
						 strcpy(nstdats[app].status, busy);
					     else
						 strcpy(nstdats[app].status, unreachable);
				            }
					 break;
				case 7 : uebnahm = no;
					 strcpy(nstdats[app].status, idle);
					 rangonce = no;
					 nstdats[app].network = 0;
					 timediff = 0;
					 break;
			}
out(nstdats[app].callingnr);
if (debug) sprintf(debs, "FFFFE: %d - %s - %s", app, nstdats[app].lastiring, inpline);
out(debs);
		}
	}
	fclose(nplogf);
}


int elen(int std, int wtag)
{
	
}


void initeinhlens()
{
	int i, j;
	j = city;
	for(i=0; i<5; i++)   einhlen[j][i] = 24000;
	for(i=5; i<9; i++)   einhlen[j][i] = 15000;
	for(i=9; i<18; i++)  einhlen[j][i] = 9000;
	for(i=18; i<21; i++) einhlen[j][i] = 15000;
	for(i=21; i<24; i++) einhlen[j][i] = 24000;
	j = reg50;
	for(i=0; i<2; i++)   einhlen[j][i] = 6000;
	for(i=2; i<5; i++)   einhlen[j][i] = 12000;
	for(i=5; i<9; i++)   einhlen[j][i] = 4500;
	for(i=9; i<12; i++)  einhlen[j][i] = 2600;
	for(i=12; i<18; i++) einhlen[j][i] = 3000;
	for(i=18; i<21; i++) einhlen[j][i] = 4500;
	for(i=21; i<24; i++) einhlen[j][i] = 6000;
	j = reg200;
	for(i=0; i<2; i++)   einhlen[j][i] = 3000;
	for(i=2; i<5; i++)   einhlen[j][i] = 12000;
	for(i=5; i<9; i++)   einhlen[j][i] = 2150;
	for(i=9; i<12; i++)  einhlen[j][i] = 1200;
	for(i=12; i<18; i++) einhlen[j][i] = 1350;
	for(i=18; i<21; i++) einhlen[j][i] = 2150;
	for(i=21; i<24; i++) einhlen[j][i] = 3000;
	j = fern;
	for(i=0; i<2; i++)   einhlen[j][i] = 2500;
	for(i=2; i<5; i++)   einhlen[j][i] = 12000;
	for(i=5; i<9; i++)   einhlen[j][i] = 2000;
	for(i=9; i<12; i++)  einhlen[j][i] = 1150;
	for(i=12; i<18; i++) einhlen[j][i] = 1250;
	for(i=18; i<21; i++) einhlen[j][i] = 2000;
	for(i=21; i<24; i++) einhlen[j][i] = 2500;
}


void initdata()
{
	int i, j;
	FILE *cfgf;
	static char *envstr;
	
	strcpy(fmeld, "");
	uebnahm = no;
	continuous = no;
	compdial = no;
	msecs = stdmsecs;
	delay.tv_sec = msecs / 1000;
	delay.tv_usec = (msecs % 1000) * 1000;
	strcpy(nstdats[Anlage].name, defname0);
	strcpy(nstdats[Buero].name, defname1);
	strcpy(nstdats[Maik].name, defname2);
	strcpy(nstdats[Felix].name, defname3);
	strcpy(nstdats[Faxmodem].name, defname4);
	strcpy(nstdats[Dummy].name, defname5);
	strcpy(normalfilename, defnormalfilename);
	strcpy(debugfilename, defdebugfilename);
	strcpy(phofilename, defphofilename);
	strcpy(denyfilename, defdenyfilename);
	strcpy(ilogfilename, defisdnlogfilename);
	strcpy(vorwahl, defvorwahl);
	pfennig = defpfennig;
	for(i=0; i<=5; i++) {
		strcpy(nstdats[i].status, idle);
		strcpy(nstdats[i].lastiring, "               ");
		strcpy(nstdats[i].lastering, "               ");
		strcpy(nstdats[i].lastenum, "");
		nstdats[i].lcall.date=101; /* 1. Januar */
		nstdats[i].lcall.units=0;
		nstdats[i].umleitruf = 0;
		nstdats[i].ringing = silence;
		strcpy(nstdats[i].lcall.duration, "--:--:--");
		strcpy(nstdats[i].lcall.betrag, "--,--");
		strcpy(nstdats[i].lcall.time, "--:--");
		strcpy(nstdats[i].lcall.number, "");
		nstdats[i].unitsum=0;
		for(j=0; j<anzlrings; j++) {
			strcpy(nstdats[i].lrings[j].time, "");
			strcpy(nstdats[i].lrings[j].number, "");
		}
		nstdats[i].network = 0;
	}
	envstr = (char *)getenv("COLUMNS");
	if (envstr != NULL)
		uppernst = (atoi(envstr) < 95) ? 4 : 5;
	else
		uppernst = 4;
	timer = yes;
	debug = no;
	isdnlogging = no;
	debdelay = no;
	lastcallmonth = 0;
	noclock = 0;
	shownames = yes;
	four = five = no;
	denyentries = no;
	defaultvalues = no;
	accepted = no;
	both = no;
	angen = no;
	othercfgfile = no;
	strcpy(cfgfilename, defcfgfilename);
	initeinhlens();
	skipcol = 0;
	bellring = 0;
	rangonce = no;
	NUMBEROFNETWORKS = sizeof(networks) / sizeof(networks[0]);
	strcpy(mailfilename, "/var/spool/mail/");
	strcat(mailfilename, getlogin());
	staticmail = no;
	mailwaiting = no;
}


pz(char outz[], unsigned int betrag)
{
	sprintf(outz, "%3d,%2d", betrag/100, betrag%100);
	if (betrag%100 < 10) outz[4] = '0';
}


pvier(char outz[], unsigned int zahl)
{
	sprintf(outz, "%4d", zahl);
	if (zahl < 1000) outz[0] = '0';
}


pzwei(char outz[], int zahl)
{
	sprintf(outz, "%2d", zahl);
	if (zahl < 10) outz[0] = '0';
}


int numtoname(char outn[], char inum[], int len) /* return value: network */
{
	int j, k, ll, li, found = no;
	int ntnet, maxnlen;
	char cmpnum[25];
	
	ntnet = 0;
	if (shownames) {
		if (strcmp("(unknown)", inum)) {
			maxnlen = 15;
			if(!strncmp(inum, "010", 3)) { /* anderer Betreiber? 010xx wegnehmen */
				maxnlen = 10;
				ntnet = (inum[3]-'0')*10+inum[4]-'0';
				for(j=5; inum[j]!='\0'; j++)
					inum[j-5]=inum[j];
				inum[j-5]='\0';
			}
			for(j=0; j<calldbsize && !found; j++) { /* versuchen, zu ersetzen */
				if(!strncmp(callers[j].number, inum, maxnlen)) {
					strncpy(outn, callers[j].name, len);
					outn[len]='\0';
					found = yes;
				}
			}
			if (!found) {
				ll = strlen(vorwahl); li = strlen(inum); /* lokale vorwahl wegnehmen und nochmal suchen */
				if (!strncmp(inum, vorwahl, ll)) {
					for(k=ll; k<li; k++)
						inum[k-ll]=inum[k];
					inum[li-ll] = '\0';
				}
				for(j=0; j<calldbsize && !found; j++) {
					if(!strcmp(callers[j].number, inum)) {
						strncpy(outn, callers[j].name, len);
						outn[len]='\0';
						found = yes;
					}
				}
			}
		}
		if (!found) {
			strncpy(outn, inum, len);
			outn[len] = '\0';
		}
	} else {
		strncpy(outn, inum, len);
		outn[len]='\0';
	}
	return ntnet;
}


void usage(int n)
{
	if (n<=2) printf("\ntkstat version %s (%s)\n\n", version, verdate);
	switch (n) {
	case 1 : printf("Usage: tkstat [options]\n\n"
			"\t-c        continuous status displaying\n"
			"\t-n        don't substitute numbers with names\n"
			"\t-a        list accepted calls instead of refused\n"
			"\t-l        list accepted and refused calls\n"
			"\t-t        don't display timer during connects\n"
			"\t-u        don't display clock\n"
			"\t-m        don't oscillate mail character\n"
			"\t-V        display version number and exit\n"
			"\t-y        use denyfile of numbers not to log\n"
			"\t-v        use debugfile instead of logfile and display LOTS of\n"
			"\t          debugging information (not recommended for normal use)\n"
			"\t-4        force 4 columns even if COLUMNS > 94\n"
			"\t-5        force 5 columns even if COLUMNS < 95\n"
			"\t-e        use default values instead of configfile\n"
			"\t-f        specify alternate config file (default is /etc/tkstat.cfg)\n"
			"\t-i        read also output file of isdnlog 2.0b\n"
			"\t-s #      skip specified column, instead list app 5 on 80char-screen\n"
			"\t-b #      ring terminal bell if incoming call on app #\n"
			"\t-d delay  change file check delay (normally 500 msecs)\n\n");
		 break;
	}
}


lcase(char s[])
{
	int i;
	
	for(i=0; i<strlen(s); i++)
		s[i] = tolower(s[i]);
}


void killhandler()
{
	endwin();
	exit(0);
}


void readphofile()
{
    FILE *fp;
    char inpline[81];

    if((fp = fopen(phofilename, "r")) == NULL) {
	fprintf(stderr, "Error: phonefile %s not found!\n"
		"Not using number/name substitution (assuming -t)\n", phofilename);
	shownames = no;
    } else {
	calldbsize = 0;
	while((fgets(inpline, 80, fp))!=NULL) {
	    sscanf(inpline, "%s %s\n", callers[calldbsize].number, callers[calldbsize].name);
	    calldbsize++;
	}
	fclose(fp);
    }
}


void readdenyfile()
{
    FILE *fp;
    char inpline[81];

    if((fp = fopen(denyfilename, "r")) == NULL) {
	fprintf(stderr, "Error: deny-phonefile %s not found!\n"
		"Every entry will be listed.\n", denyfilename);
	denyentries = no;
    } else {
	denydbsize = 0;
	while((fgets(inpline, 80, fp))!=NULL) {
	    sscanf(inpline, "%s\n", noentrynums[denydbsize].number);
	    denydbsize++;
	}
	fclose(fp);
    }
}


void reinitphodb()
{
    readphofile();
    if (denyentries) readdenyfile();
    signal(SIGHUP, reinitphodb);
}


int main(int argc, char **argv)
{
	int optc, i;
	FILE *cfgf;
	char cfgin[255], cfgcmd[30], cfgarg[230];
	char *fileread;
	
	initdata();
	while ((optc = getopt(argc, argv, "efalv45unityVcm?d:s:b:")) != EOF) switch(optc) {
	case 'c': continuous = yes;
	    break;
	case 'u': noclock = yes;
	    break;
	case 't': timer = no;
	    break;
	case 'n': shownames = no;
	    break;
	case 'a': accepted = yes;
	    break;
	case 'l': both = yes;
	    break;
	case 'v': debug = yes;
	    break;
	case 'i': isdnlogging = yes;
	    break;
	case '5': five = yes;
	    break;
	case '4': four = yes;
	    break;
	case 'e': defaultvalues = yes;
	    break;
	case 'm': staticmail = yes;
	    break;
	case 's': skipcol = atoi(optarg);
	    break;
	case 'b': bellring = atoi(optarg);
	    break;
	case 'y': denyentries = yes;
	    break;
	case 'f': othercfgfile = yes;
	    strcpy(cfgfilename, optarg);
	    break;
	case 'd': msecs = atoi(optarg);
	    delay.tv_sec = msecs / 1000;
	    delay.tv_usec = (msecs % 1000) * 1000;
	    break;
	case '?': usage(1);
	    exit(1);
	case 'V': usage(2);
	    exit(1);
	}
	if (five) uppernst=5;
	if (four) uppernst=4;
	if (skipcol) uppernst = 5;
	if (shownames) readphofile();
	if (denyentries) readdenyfile();
	if ((cfgf=fopen(cfgfilename, "r"))!=NULL) {
		while((fileread=fgets(cfgin, 254, cfgf))!=NULL) {
			sscanf(cfgin, "%s %s\n", cfgcmd, cfgarg);
			lcase(cfgcmd);
			if(!strcmp(cfgcmd, "logfilename")) strcpy(normalfilename, cfgarg);
			if(!strcmp(cfgcmd, "debugfilename")) strcpy(debugfilename, cfgarg);
			if(!strcmp(cfgcmd, "phonefilename")) strcpy(phofilename, cfgarg);
			if(!strcmp(cfgcmd, "denyfilename")) strcpy(denyfilename, cfgarg);
			if(!strcmp(cfgcmd, "isdnlogfilename")) strcpy(ilogfilename, cfgarg);
			if(!strcmp(cfgcmd, "vorwahl")) strcpy(vorwahl, cfgarg);
			if(!strcmp(cfgcmd, "pfennig")) pfennig = atoi(cfgarg);
			if((!strncmp(cfgcmd, "name", 4)) && (strlen(cfgcmd) == 5))
				strcpy(nstdats[cfgcmd[4]-'0'].name, cfgarg);
		}
		fclose(cfgf);
	} else {
		if (othercfgfile || !defaultvalues) printf("Config file not found, using defaults\n");
		
	}
	if(debug)
		strcpy(logfilename, debugfilename);
	else
		strcpy(logfilename, normalfilename);
out("ifdebcont");
	if(debug&&continuous)
		debfile=fopen(debfilename, "a");
out("ifcont");
	if (continuous) {
		initscr();
		leaveok(curscr, TRUE);
		if(signal(SIGTERM, killhandler) == SIG_ERR) {
			printf("Error initializing kill (15) signal routine\n");
			exit(3);
		}
		if(signal(SIGHUP, reinitphodb) == SIG_ERR) {
			printf("Error initializing kill (1) signal routine\n");
			exit(4);
		}
		getlogdata();
		endwin();
	} else {
		getlogdata();
if(debug) {
    for(i=0;i<=5;i++) sprintf(debs, "FFFFF: %s\n", nstdats[i].lastiring);
    out(debs);
}
		printstats();
	}
	if(debug&&continuous)
		fclose(debfile);
	exit(0);
}
